
# q-expansion of newform 320.4.a.j, downloaded from the LMFDB on 29 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 320
weight = 4
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [4], [-5], [-16], [36], [42], [-110], [-116], [-16], [-198], [-240], [258], [442], [-292], [-392], [-142], [-348], [570], [692], [-168], [-134], [-784], [564], [1034], [-382], [674], [992], [-500], [-1046], [-558], [328], [-212], [1434], [2196], [2418], [-3672], [-358], [2564], [3056], [234], [524], [1138], [-1520], [-2142], [2306], [-3288], [-3876], [5688], [-2796], [-4446], [2522], [-816], [-5422], [-5900], [5250], [-6240], [714], [-2144], [4466], [-5302], [-6932], [4034], [-3836], [-664], [2986], [-2726], [-9212], [-3278], [4956], [-4678], [1890], [6472], [-1960], [-8750], [-380], [9688], [-3870], [-1622], [9906], [-4214], [-7012], [1602], [3584], [-3470], [3416], [9708], [-10366], [-16742], [1258], [-13528], [6916], [-1728], [-16656], [-1084], [5804], [-10512], [4314], [-1190], [-3780], [11002], [5908], [-14806], [-684], [-2582], [-2540], [22786], [7884], [-21902], [-15080], [-19702], [-7320], [-24350], [19546], [3476], [-21880], [20994], [-18204], [2064], [-9942], [24236], [-17614], [13058], [33186], [-31716], [-2084], [7418], [18242], [-3024], [26176], [17818], [-22052], [-15840], [-21024], [38034], [37802], [15042], [-5950], [23364], [-19846], [24762], [16644], [-3182], [7504], [12604], [-12230], [-9656], [-5806], [-39094], [-18876], [-32296], [9578], [-41710], [2260], [33696], [7756], [5312], [23576], [-19038], [20570], [21386], [38020], [20202], [-29840], [-12476], [-36974], [16368], [49552], [-24414], [-63214], [-5838], [-8060], [13210], [-10376], [4090], [39184], [30666], [27252], [59298], [48192], [18154], [17384], [-27412], [-18814], [-6918], [17928], [24850], [-20198], [7172], [41722], [-75600], [-27326], [46812], [40444], [-26214], [-844], [48986], [-29870], [7898], [-14142], [-63040], [-28502], [-76256], [9554], [-33118], [25076], [-67398], [45568], [-44060], [-10470], [-30332], [-21166], [-6542], [28208], [26876], [-28760], [15418], [-22024], [-41870], [-67552], [-77190], [22594], [-32840], [-36030], [72712], [57604], [-105550], [72618], [-101664], [-64432], [58020], [35722], [44780], [67600], [5722], [64092], [53784], [72370], [-76590], [-82460], [101528], [1044], [-49036], [65088], [76714], [-62606], [12024], [-50776], [63468], [70260], [-22328], [-28006], [-107070], [-26800], [9242], [-90934], [-97140], [-98350], [38620], [52098], [-46678], [57848], [40436], [-33598], [128026], [-69886], [110908], [89290], [-45590], [-29972], [-125918], [-90326], [-73596], [-38486], [-32416], [-1134], [-128592], [-96036], [1050], [-66534], [-6948], [86040], [-20200], [145376], [53186], [-89780], [44928], [117714], [-93038], [-12520], [17250], [94826], [138084], [-75318], [142564], [-33526], [21082], [-11632], [146706], [34116], [-77356], [-63398], [-100982], [-70096], [17940], [79188], [-113086], [77804], [55626], [22616], [-144926], [-118808], [-129774], [-103966], [-122396], [-44048], [-36614], [85036], [-78672], [-147390], [145074], [-60676], [-32886], [-22566], [-10056], [81274], [171858], [-117860], [-12340], [110760], [-115358], [86186], [171514], [3104], [3332], [129124], [169052], [10650], [-177438], [51130], [-40680], [168914], [14250], [177698], [167320], [-100390], [-90692], [-88958], [-123572], [197792], [-16974], [112556], [8794], [-207990], [-60536], [162514], [65194], [46464], [-21724], [-202382], [118832], [161602], [89562], [57528], [-5308], [18292], [-5542], [-72822], [-50912], [43850], [7308], [-193900], [-187192], [3794], [5512], [135002], [210444], [69456], [101346], [166098], [220394], [-155430], [-100070], [-259776], [-51294], [28412], [44224], [-193088], [-102510], [20332], [190792], [-222270], [114722], [-236204], [-176828], [272360], [-112566], [-71872], [-6598], [-57916], [216530], [-158854], [-71070], [-182856], [114698], [-11454], [-49672], [261386], [186834], [-5612], [-138756], [143730], [-199374], [281068], [-132980], [250778], [203210], [-111936], [158800], [-25902], [-140976], [-41126], [-224926], [43896], [-168028], [168538], [34122], [93268], [144522], [143668], [-6312]]
